import django.apps import AppConfig


class TestFieldsApp(AppConfig):
    name = 'fields'
    label = 'fields'


class TestShortcutsApp(AppConfig):
    name = 'shortcuts_app'
    label = 'shortcuts_app'


class TestBaseApp(AppConfig):
    name = 'base'
    label = 'base'
